'''
LMS server
based on https://github.com/molobrakos/lms
'''

import logging
from requests import Session
from datetime import timedelta
from sys import version_info
from urllib.parse import urlparse,urlunparse,unquote, quote

from lmsmusly.lms.player import LMSPlayer
from lmsmusly.utils import config

if version_info < (3, 0):
	exit('Python 3 required')

__version__ = '0.0.1'

_LOGGER = logging.getLogger(__name__)
TIMEOUT = timedelta(seconds=config.LMS_TIMEOUT)


class LMSServer:

	def __init__(self, host=None, port=9000, username=None, password=None):
		self._host = host
		self._port = port
		self._session = Session()
		self._username = username
		self._password = password
		self._state = {}
		self._players = {}
		if username and password:
			self._session.auth = (username, password)

	def __str__(self):
		return '%s:%d (%s)' % (
			self._host, self._port, self.version) + '\n' + '\n'.join(
				'\t - player: ' + str(p) for p in self.players)

	def query(self, *command, player=''):
		url = 'http://{}:{}/jsonrpc.js'.format(self._host, self._port)
		data = dict(id='1',
					method='slim.request',
					params=[player, command])
		#_LOGGER.debug('URL: %s Data: %s', url, data)
		try:
			result = self._session.post(
				url, json=data, timeout=TIMEOUT.seconds)
			result.raise_for_status()
			result = result.json()
#			_LOGGER.debug(result)
			return result['result']
		except OSError as e:
			_LOGGER.warning('Could not query server: {}'.format( e))
			_LOGGER.debug('Data: {}'.format(repr( data)))
			return {}

	def update(self):
		self._state = self.server_status
		self._players = {player['playerid']: LMSPlayer(self, player)
						 for player in self._state['players_loop']}
		#_LOGGER.debug("LMSserver players: {}".format( repr(self._players)))
		self.update_players()

	def update_players(self):
		for player in self.players:
			player.update()

	@property
	def _url(self):
		if self._username and self._password:
			return 'http://{username}:{password}@{server}:{port}/'.format(
				username=self._username,
				password=self._password,
				server=self._host,
				port=self._port)
		return 'http://{server}:{port}/'.format(
			server=self._host,
			port=self._port)

	@property
	def version(self):
		return self._state.get('version')

	@property
	def server_status(self):
		return self.query('serverstatus', '-')

	@property
	def players_status(self):
		return self.query('players', 'status')

	@property
	def players(self):
		return self._players.values()

	def player(self, player_id):
		return self._players[player_id]
	
	def is_player_known(self, player_id):
		return player_id in self._players

	def getSonginfoForPath(self, file, tags='ud'):
		url = urlunparse( ('file', '', quote( file), '', '', '') )
		return self.query('songinfo', 0, 1, 'url:{url} tags:{tags}'.format(url=url, tags=tags))

